@@ -4,7 +4,7 @@ use 5.008_001;
 use strict;
 use warnings;
 
-our $VERSION = '0.1047';
+our $VERSION = '0.1048';
 
 use Carp       ();
 use File::Spec ();
@@ -48,8 +48,8 @@ use Text::Xslate::Util qw(
 );
 
 BEGIN {
-    my $dump_load_file = scalar($DEBUG =~ /\b dump=load_file \b/xms);
-    *_DUMP_LOAD_FILE = sub(){ $dump_load_file };
+    my $dump_load = scalar($DEBUG =~ /\b dump=load \b/xms);
+    *_DUMP_LOAD = sub(){ $dump_load };
 
     *_ST_MTIME = sub() { 9 }; # see perldoc -f stat
 
@@ -64,12 +64,14 @@ my $IDENT   = qr/(?: [a-zA-Z_][a-zA-Z0-9_\@]* )/xms;
 # version-path-{compiler options}
 my $XSLATE_MAGIC    = qq{.xslate "%s-%s-{%s}"\n};
 
+# the real defaults are dfined in the parser
 my %parser_option = (
     line_start => undef,
     tag_start  => undef,
     tag_end    => undef,
 );
 
+# the real defaults are defined in the compiler
 my %compiler_option = (
     syntax     => undef,
     escape     => undef,
@@ -93,7 +95,7 @@ sub options { # overridable
         suffix       => '.tx',
         path         => ['.'],
         input_layer  => ':utf8',
-        cache        => 1,
+        cache        => 1, # 0: not cached, 1: checks mtime, 2: always cached
         cache_dir    => _DEFAULT_CACHE_DIR,
         module       => undef,
         function     => undef,
@@ -186,23 +188,22 @@ sub load_string { # for <string>
     }
     $self->{string_buffer} = $string;
     my $asm = $self->compile($string);
-    $self->_assemble($asm, undef, undef, undef, undef);
+    $self->_assemble($asm, '<string>', \$string, undef, undef);
     return $asm;
 }
 
 sub find_file {
-    my($self, $file, $threshold_mtime) = @_;
+    my($self, $file) = @_;
 
     my $fullpath;
     my $cachepath;
     my $orig_mtime;
     my $cache_mtime;
-
     foreach my $p(@{$self->{path}}) {
-        if(ref $p eq 'HASH') {
+        if(ref $p eq 'HASH') { # virtual path
             defined(my $content = $p->{$file}) or next;
             $fullpath   = \$content;
-            $orig_mtime = 0; # always fresh
+            $orig_mtime = $^T;
         }
         else {
             $fullpath = File::Spec->catfile($p, $file);
@@ -212,17 +213,8 @@ sub find_file {
 
         # $file is found
 
-        $cachepath = File::Spec->catfile($self->{cache_dir}, $file . 'c');
-
-        if(-f $cachepath) {
-            my $cmt = (stat(_))[_ST_MTIME]; # compiled
-
-            # see also tx_load_template() in xs/Text-Xslate.xs
-            if(($threshold_mtime || $cmt) >= $orig_mtime) {
-                $cache_mtime = $cmt;
-            }
-        }
-
+        $cachepath   = File::Spec->catfile($self->{cache_dir}, $file . 'c');
+        $cache_mtime = (stat($cachepath))[_ST_MTIME]; # may fail, but doesn't matter
         last;
     }
 
@@ -230,7 +222,7 @@ sub find_file {
         $self->_error("LoadError: Cannot find '$file' (path: @{$self->{path}})");
     }
 
-    print STDOUT "  find_file: $fullpath (", ($cache_mtime || 0), ")\n" if _DUMP_LOAD_FILE;
+    print STDOUT "  find_file: $fullpath (", ($cache_mtime || 0), ")\n" if _DUMP_LOAD;
 
     return {
         fullpath    => $fullpath,
@@ -244,13 +236,13 @@ sub find_file {
 sub load_file {
     my($self, $file, $mtime) = @_;
 
-    print STDOUT "load_file($file)\n" if _DUMP_LOAD_FILE;
+    print STDOUT "load_file($file)\n" if _DUMP_LOAD;
 
     if($file eq '<string>') { # simply reload it
         return $self->load_string($self->{string_buffer});
     }
 
-    my $fi = $self->find_file($file, $mtime);
+    my $fi = $self->find_file($file);
 
     my $asm = $self->_load_compiled($fi, $mtime) || $self->_load_source($fi, $mtime);
 
@@ -317,7 +309,7 @@ sub _load_source {
             Carp::carp("Xslate: Cannot open $cachepath for writing (ignored): $!");
         }
     }
-    if(_DUMP_LOAD_FILE) {
+    if(_DUMP_LOAD) {
         printf STDERR "  _load_source: cache(%s)\n",
             defined $fi->{cache_mtime} ? $fi->{cache_mtime} : 'undef';
     }
@@ -325,17 +317,27 @@ sub _load_source {
     return $asm;
 }
 
+# load compiled templates if they are fresh enough
 sub _load_compiled {
-    my($self, $fi, $threshold_mtime) = @_;
+    my($self, $fi, $threshold) = @_;
 
-    $fi->{cache_mtime} = undef if $self->{cache} == 0;
-
-    return undef if !$fi->{cache_mtime};
-
-    $threshold_mtime ||= $fi->{cache_mtime};
+    if($self->{cache} >= 2) {
+        # threshold is the most latest modified time of all the related caches,
+        # so if the cache level >= 2, they seems always fresh.
+        $threshold = '+inf';
+    }
+    else {
+        $threshold ||= $fi->{cache_mtime};
+    }
+    # see also tx_load_template() in xs/Text-Xslate.xs
+    if(!( defined($fi->{cache_mtime}) and $self->{cache} >= 1
+            and $threshold >= $fi->{orig_mtime} )) {
+        printf "  _load_compiled: no fresh cache: %s", Text::Xslate::Util::p($fi) if _DUMP_LOAD;
+        $fi->{cache_mtime} = undef;
+        return undef;
+    }
 
     my $cachepath = $fi->{cachepath};
-
     open my($in), '<' . $self->{input_layer}, $cachepath
         or $self->_error("LoadError: Cannot open $cachepath for reading: $!");
 
@@ -376,11 +378,11 @@ sub _load_compiled {
                 $dep_mtime = '+inf'; # force reload
                 Carp::carp("Xslate: Failed to stat $value (ignored): $!");
             }
-            if($dep_mtime > $threshold_mtime){
+            if($dep_mtime > $threshold){
                 printf "  _load_compiled: %s(%s) is newer than %s(%s)\n",
                     $value,     scalar localtime($dep_mtime),
-                    $cachepath, scalar localtime($threshold_mtime)
-                        if _DUMP_LOAD_FILE;
+                    $cachepath, scalar localtime($threshold)
+                        if _DUMP_LOAD;
 
                 return undef;
             }
@@ -389,7 +391,7 @@ sub _load_compiled {
         push @asm, [ $name, $value, $line, $file, $symbol ];
     }
 
-    if(_DUMP_LOAD_FILE) {
+    if(_DUMP_LOAD) {
         printf STDERR "  _load_compiled: cache(%s)\n",
             defined $fi->{cache_mtime} ? $fi->{cache_mtime} : 'undef';
     }
@@ -481,7 +483,7 @@ Text::Xslate - High performance template engine
 
 =head1 VERSION
 
-This document describes Text::Xslate version 0.1047.
+This document describes Text::Xslate version 0.1048.
 
 =head1 SYNOPSIS
 
@@ -519,21 +521,10 @@ This document describes Text::Xslate version 0.1047.
 
     print $tx->render_string($template, \%vars);
 
-    # you can tell the engine that some strings are already escaped.
-    use Text::Xslate qw(mark_raw html_escape);
-
-    $vars{email} = mark_raw('gfx &lt;gfuji at cpan.org&gt;');
-    # or
-    # $vars{email} = html_escape('gfx <gfuji at cpan.org>');
-
-    # if you want Template-Toolkit syntx:
-    $tx = Text::Xslate->new(syntax => 'TTerse');
-    # ...
-
 =head1 DESCRIPTION
 
-B<Text::Xslate> (pronounced as /eks-leit/) is a high performance template engine
-tuned for persistent applications.
+B<Text::Xslate> is a high performance template engine tuned for persistent
+applications.
 This engine introduces the virtual machine paradigm. Templates are
 compiled into xslate intermediate code, and then executed by the xslate
 virtual machine.
@@ -582,7 +573,7 @@ Here is a result of F<benchmark/others.pl> to compare various template engines.
 You can see Xslate is 36 times faster than Template-Toolkit, and 4 times faster
 than HTML::Template::Pro and Text::ClearSilver, which are implemented in XS.
 
-=head3 Auto HTML escaping
+=head3 Auto escaping to HTML meta characters
 
 All the template expressions the engine interpolates into templates are
 html-escaped automatically, so the output has no possibility to XSS by default.
@@ -705,6 +696,9 @@ Specifies the escape mode, which is automatically applied to template expression
 
 Possible escape modes are B<html> and B<none>.
 
+Note that C<none> mode is provided for non-HTML templates, e.g. mail generators,
+so you must not to use it for HTML templates because it is unsafe.
+
 This option is passed to the compiler directly.
 
 =item C<< line_start => $token // $parser_defined_str >>
@@ -719,7 +713,7 @@ Specify the token to start inline code as a string, which C<quotemeta> will be a
 
 This option is passed to the parser via the compiler.
 
-=item C<< line_start => $str // $parser_defined_str >>
+=item C<< tag_end => $str // $parser_defined_str >>
 
 Specify the token to end inline code as a string, which C<quotemeta> will be applied to.
 
@@ -751,7 +745,7 @@ Note that I<$file> may be cached according to the cache level.
 Renders a template string with variables, and returns the result.
 I<\%vars> is optional.
 
-Note that I<$string> is never cached.
+Note that I<$string> is never cached, so this may be not suitable for web applications.
 
 =head3 B<< $tx->load_file($file) :Void >>
 
@@ -770,7 +764,7 @@ Here is an example to to load all the templates which is in a given path:
     find sub {
         if(/\.tx$/) {
             my $file = $File::Find::name;
-            $file =~ s/\Q$path\E .//xsm;
+            $file =~ s/\Q$path\E .//xsm; # fix path names
             $tx->load_file($file);
         }
     }, $path;
@@ -780,17 +774,17 @@ Here is an example to to load all the templates which is in a given path:
 
 =head3 B<< Text::Xslate->current_engine :XslateEngine >>
 
-Returns the current Xslate engine while executing. Otherwise, returns C<undef>.
+Returns the current Xslate engine while executing. Otherwise returns C<undef>.
 This method is significant when it is called by template functions and methods.
 
-=head3 B<< Text::Xslate->current_file :XslateEngine >>
+=head3 B<< Text::Xslate->current_file :Str >>
 
-Returns the current file name while executing. Otherwise, returns C<undef>.
+Returns the current file name while executing. Otherwise returns C<undef>.
 This method is significant when it is called by template functions and methods.
 
-=head3 B<< Text::Xslate->current_line :XslateEngine >>
+=head3 B<< Text::Xslate->current_line :Int >>
 
-Returns the current line number while executing. Otherwise, returns C<undef>.
+Returns the current line number while executing. Otherwise returns C<undef>.
 This method is significant when it is called by template functions and methods.
 
 =head2 Exportable functions
@@ -805,7 +799,7 @@ For example:
     my $tx   = Text::Xslate->new();
     my $tmpl = 'Mailaddress: <: $email :>';
     my %vars = (
-        email => mark_raw('Foo &lt;foo@example.com&gt;'),
+        email => mark_raw('Foo &lt;foo at example.com&gt;'),
     );
     print $tx->render_string($tmpl, \%email);
     # => Mailaddress: Foo &lt;foo@example.com&gt;
@@ -831,7 +825,7 @@ expressions.
 This function is available in templates as the C<html> filter, but you'd better
 to use C<unmark_raw> to ensure expressions to be html-escaped.
 
-=head2 Application
+=head2 Command line interface
 
 The C<xslate(1)> command is provided as a CLI to the Text::Xslate module,
 which is used to process directory trees or to evaluate one liners.
@@ -923,15 +917,17 @@ L<Text::Xslate::Syntax::Metakolon>
 
 L<Text::Xslate::Syntax::TTerse>
 
-Cookbook:
+Documents:
 
-L<Text::Xslate::Cookbook>
+L<Text::Xslate::Manual::Cookbook>
+
+L<Text::Xslate::Manual::FAQ>
 
 Xslate command:
 
 L<xlsate>
 
-Other template modules:
+Other template modules that Xslate is influenced by:
 
 L<Text::MicroTemplate>
 
