@@ -65,15 +65,12 @@ my %binary = (
 );
 my %logical_binary = (
     '&&'  => 'and',
-    'and' => 'and',
     '||'  => 'or',
-    'or'  => 'or',
     '//'  => 'dor',
 );
 
 my %unary = (
     '!'   => 'not',
-    'not' => 'not',
     '+'   => 'noop',
     '-'   => 'minus',
     '+^'  => 'bitneg',
@@ -768,14 +765,14 @@ sub _prepare_cond_expr {
     my $t = "and";
     my $f = "or";
 
-    while(any_in($expr->id, "!", "not")) {
+    while($expr->id eq '!') {
         $expr    = $expr->first;
         ($t, $f) = ($f, $t);
     }
 
     if($expr->is_logical and any_in($expr->id, qw(== !=))) {
         my $rhs = $expr->second;
-        if($rhs->arity eq "literal" and $rhs->id eq "nil") {
+        if($rhs->arity eq "nil") {
             # add prefix 'd' (i.e. "and" to "dand", "or" to "dor")
             substr $t, 0, 0, 'd';
             substr $f, 0, 0, 'd';
@@ -880,7 +877,7 @@ sub _generate_given {
 sub _generate_variable {
     my($self, $node) = @_;
 
-    if(defined(my $lvar_id = $self->lvar->{$node->id})) {
+    if(defined(my $lvar_id = $self->lvar->{$node->value})) {
         return $self->opcode( load_lvar => $lvar_id, symbol => $node );
     }
     else {
@@ -900,17 +897,15 @@ sub _generate_marker {
 
 sub _generate_literal {
     my($self, $node) = @_;
-
-    my $value = $node->value;
-    if(defined $value){
-        return $self->opcode( literal => $value );
-    }
-    else {
-        return $self->opcode( 'nil' );
-    }
+    return $self->opcode( literal => $node->value );
 }
 
-sub _generate_objectliteral {
+sub _generate_nil {
+    my($self) = @_;
+    return $self->opcode('nil');
+}
+
+sub _generate_composer {
     my($self, $node) = @_;
 
     my $list = $node->first;
@@ -949,10 +944,11 @@ sub _generate_field {
     my $field = $node->second;
 
     # $foo.field
-    if($field->arity eq "word") {
+    # $foo["field"]
+    if($field->arity eq "literal") {
         return
             @lhs,
-            $self->opcode( fetch_field_s => $field->id );
+            $self->opcode( fetch_field_s => $field->value );
     }
     # $foo[expression]
     else {
@@ -1155,7 +1151,7 @@ sub _localize_vars {
 sub _variable_to_value {
     my($self, $arg) = @_;
 
-    my $name = $arg->id;
+    my $name = $arg->value;
     $name =~ s/\$//;
     return $name;
 }
@@ -1183,7 +1179,7 @@ sub _fold_constants {
 
     my $result = eval {
         my @tmp_code = (@{$code}, $self->opcode('print_raw'), $self->opcode('end'));
-        $engine->_assemble(\@tmp_code, undef, undef, undef, undef);
+        $engine->_assemble(\@tmp_code, '<string>', undef, undef, undef);
         $engine->render(undef);
     };
     if($@) {
